

ALTER PROCEDURE [dbo].[SoftwareCountByComputer]
	@RDomDN			nvarchar(101) = '%',
	@ROUDN			nvarchar(401) = '%',
	@DN				nvarchar(401) = '%',
	@SoftwareName		nvarchar(500) = '%',
	@ComputerCountMin	int = NULL,
	@ComputerCountMax	int = 0,
	@UserCountMin		int = NULL,
	@UserCountMax		int = 0
AS
	SELECT	LTRIM(RTRIM(sv.SoftwareName)) AS SoftwareName,
		COUNT(DISTINCT sv.Version) AS VersionCount,
		COUNT(so.ComputerID) as ComputerInstallations,
		COUNT(DISTINCT so.ComputerID) as ComputerCount,
		SUM(CASE WHEN so.UserID <> 1 THEN 1 ELSE 0 END) as UserInstallCount,
		COUNT(DISTINCT CASE WHEN so.UserID <> 1 THEN so.UserID ELSE NULL END) as UserCount
	FROM	(SELECT * FROM ComputerDirectory
		 WHERE	OUDN LIKE @DN AND OURDN LIKE @ROUDN AND DomainDN LIKE @RDomDN
		) AS cd,
		SoftwareOccurrence AS so,
		SoftwareVersion AS sv
	WHERE	dbo.LikeNullable(LTRIM(RTRIM(sv.SoftwareName)), @SoftwareName) = 1
	  AND	cd.ComputerID = so.ComputerID
	  AND	so.SoftwareID = sv.SoftwareID
	GROUP BY sv.SoftwareName
	HAVING	dbo.RangeMatchInt(
			COUNT(DISTINCT so.ComputerID),
			@ComputerCountMin,
			@ComputerCountMax
		) = 1
	  AND	dbo.RangeMatchInt(
	  		COUNT(DISTINCT CASE WHEN so.UserID <> 1 THEN so.UserID ELSE NULL END),
			@UserCountMin,
			@UserCountMax
		) = 1
	ORDER BY sv.SoftwareName
GO

ALTER PROCEDURE [dbo].[ComputerBySoftware]
	@RDomDN			nvarchar(101) = '%',
	@ROUDN			nvarchar(401) = '%',
	@DN				nvarchar(401) = '%', 
	@SoftwareName	nvarchar(500) ,
	@Version	   	nvarchar(64) 
AS
SELECT  
		LTRIM(RTRIM(sv.SoftwareName)) AS SoftwareName,
		LTRIM(RTRIM(sv.Version)) AS Version,
		cd.ComputerID,
		cd.ComputerCN,
		cd.OURDN AS OrganizationalUnit,
		cd.DomainDN AS DomainDN

FROM  SoftwareOccurrence so
INNER JOIN ComputerDirectory cd ON so.ComputerID = cd.ComputerID AND cd.OUDN LIKE @DN AND cd.OURDN LIKE @ROUDN AND cd.DomainDN LIKE @RDomDN
INNER JOIN SoftwareVersion sv ON so.SoftwareID = sv.SoftwareID AND (sv.SoftwareName IS NULL OR sv.SoftwareName LIKE @SoftwareName) AND (sv.Version IS NULL OR sv.Version LIKE @Version)
GO

ALTER PROCEDURE [dbo].[SoftwareVersionCountByComputer]
	@RDomDN			nvarchar(101) = '%',
	@ROUDN			nvarchar(401) = '%',
	@DN				nvarchar(401) = '%',
	@SoftwareName		nvarchar(500) = '%',
	@Version	   	nvarchar(64) = '%',
	@ComputerCountMin	int = 0,
	@ComputerCountMax	int = NULL,
	@UserCountMin		int = 0,
	@UserCountMax		int = NULL
AS
CREATE TABLE #Software (SoftwareID int, SoftwareName nvarchar(500),  [Version] nvarchar(500))
INSERT INTO #Software
SELECT SoftwareId, SoftwareName, [Version] FROM SoftwareVersion WHERE SoftwareName LIKE @SoftwareName AND  [Version] LIKE @Version

 

 IF (@RDomDN='%')
BEGIN

SELECT * FROM
(
SELECT	DISTINCT  LTRIM(RTRIM(sv.SoftwareName)) AS SoftwareName
					, LTRIM(RTRIM(sv.[Version])) AS Version
					, ComputerCNT.cnt  as ComputerCount
					, UserInstallCNT.cnt as UserInstallCount
					, UserCNT.cnt as UserCount

FROM	
	ComputerDirectory cd	

		INNER JOIN SoftwareOccurrence so ON cd.ComputerID = so.ComputerID AND cd.OUDN LIKE @DN AND cd.OURDN LIKE @ROUDN
		INNER JOIN  #Software sv ON so.SoftwareID = sv.SoftwareID

		INNER JOIN  (SELECT socnt.SoftwareID , COUNT(DISTINCT CASE WHEN socnt.UserID <> 1 THEN 1 ELSE NULL END) cnt FROM SoftwareOccurrence socnt
		GROUP BY SoftwareID)  UserCNT	ON UserCNT.SoftwareID = so.SoftwareID

		INNER JOIN  (SELECT socnt.SoftwareID , SUM(CASE WHEN socnt.UserID <> 1 THEN 1 ELSE 0 END) cnt FROM SoftwareOccurrence socnt
		GROUP BY SoftwareID)  UserInstallCNT	ON UserInstallCNT.SoftwareID = so.SoftwareID
		
		INNER JOIN  (SELECT socnt.SoftwareID , COUNT(DISTINCT socnt.ComputerID) cnt FROM SoftwareOccurrence socnt
		GROUP BY SoftwareID)  ComputerCNT	ON ComputerCNT.SoftwareID = so.SoftwareID
 
		
	WHERE	((@ComputerCountMax IS NULL AND ComputerCNT.cnt>=@ComputerCountMin) OR (ComputerCNT.cnt BETWEEN @ComputerCountMin AND @ComputerCountMax))
	  AND	((@UserCountMax     IS NULL AND UserCNT.cnt>=@UserCountMin) OR (UserCNT.cnt	 BETWEEN @UserCountMin     AND @UserCountMax	))
) CountData	

ORDER BY CountData.SoftwareName, CountData.Version

END
ELSE BEGIN


SELECT * FROM
(
SELECT	DISTINCT  LTRIM(RTRIM(sv.SoftwareName)) AS SoftwareName
					, LTRIM(RTRIM(sv.[Version])) AS Version
					, ComputerCNT.cnt  as ComputerCount
					, UserInstallCNT.cnt as UserInstallCount
					, UserCNT.cnt as UserCount

FROM	
	ComputerDirectory cd	

		INNER JOIN SoftwareOccurrence so ON cd.ComputerID = so.ComputerID AND cd.OUDN LIKE @DN AND cd.OURDN LIKE @ROUDN AND cd.DomainDN LIKE @RDomDN
		INNER JOIN  #Software sv ON so.SoftwareID = sv.SoftwareID
		INNER JOIN  (SELECT socnt.SoftwareID , COUNT(DISTINCT CASE WHEN socnt.UserID <> 1 THEN 1 ELSE NULL END) cnt FROM SoftwareOccurrence socnt
			  		GROUP BY SoftwareID)  UserCNT	ON UserCNT.SoftwareID = so.SoftwareID

		INNER JOIN  (SELECT socnt.SoftwareID , SUM(CASE WHEN socnt.UserID <> 1 THEN 1 ELSE 0 END) cnt FROM SoftwareOccurrence socnt
		GROUP BY SoftwareID)  UserInstallCNT	ON UserInstallCNT.SoftwareID = so.SoftwareID
		
		INNER JOIN  (SELECT socnt.SoftwareID , COUNT(DISTINCT socnt.ComputerID) cnt FROM SoftwareOccurrence socnt
		GROUP BY SoftwareID)  ComputerCNT	ON ComputerCNT.SoftwareID = so.SoftwareID
 
		
	  WHERE  	((@ComputerCountMax IS NULL AND ComputerCNT.cnt>=@ComputerCountMin) OR (ComputerCNT.cnt BETWEEN @ComputerCountMin AND @ComputerCountMax))
	  AND	((@UserCountMax     IS NULL AND UserCNT.cnt>=@UserCountMin) OR (UserCNT.cnt	 BETWEEN @UserCountMin     AND @UserCountMax	))
) CountData	

ORDER BY CountData.SoftwareName, CountData.Version

DROP TABLE #Software	

END
